@@ -1,10 +1,13 @@
+/*
+ * contrib/btree_gist/btree_gist.h
+ */
 #ifndef __BTREE_GIST_H__
 #define __BTREE_GIST_H__
 
-#include "postgres.h"
-#include "access/gist.h"
-#include "access/itup.h"
 #include "access/nbtree.h"
+#include "fmgr.h"
+
+#define BtreeGistNotEqualStrategyNumber 6
 
 /* indexed types */
 
@@ -24,21 +27,15 @@ enum gbtree_type
 	gbt_t_date,
 	gbt_t_intv,
 	gbt_t_macad,
+	gbt_t_macad8,
 	gbt_t_text,
 	gbt_t_bpchar,
 	gbt_t_bytea,
 	gbt_t_bit,
-	gbt_t_inet
+	gbt_t_bool,
+	gbt_t_inet,
+	gbt_t_uuid,
+	gbt_t_enum
 };
 
-
-
-/*
- * Generic btree functions
- */
-
-Datum		gbtreekey_in(PG_FUNCTION_ARGS);
-
-Datum		gbtreekey_out(PG_FUNCTION_ARGS);
-
 #endif
