@@ -444,7 +444,7 @@ parse_re_flags(pg_re_flags *flags, text *opts)
 					ereport(ERROR,
 							(errcode(ERRCODE_INVALID_PARAMETER_VALUE),
 							 errmsg("invalid regular expression option: \"%.*s\"",
-									pg_mblen_range(opt_p + i, opt_p + opt_len), opt_p + i)));
+									pg_mblen(opt_p + i), opt_p + i)));
 					break;
 			}
 		}
@@ -674,13 +674,12 @@ textregexreplace(PG_FUNCTION_ARGS)
 	if (VARSIZE_ANY_EXHDR(opt) > 0)
 	{
 		char	   *opt_p = VARDATA_ANY(opt);
-		const char *end_p = opt_p + VARSIZE_ANY_EXHDR(opt);
 
 		if (*opt_p >= '0' && *opt_p <= '9')
 			ereport(ERROR,
 					(errcode(ERRCODE_INVALID_PARAMETER_VALUE),
 					 errmsg("invalid regular expression option: \"%.*s\"",
-							pg_mblen_range(opt_p, end_p), opt_p),
+							pg_mblen(opt_p), opt_p),
 					 errhint("If you meant to use regexp_replace() with a start parameter, cast the fourth argument to integer explicitly.")));
 	}
 
@@ -774,15 +773,12 @@ similar_escape_internal(text *pat_text, text *esc_text)
 			   *r;
 	int			plen,
 				elen;
-	const char *pend;
 	bool		afterescape = false;
+	bool		incharclass = false;
 	int			nquotes = 0;
-	int			bracket_depth = 0;	/* square bracket nesting level */
-	int			charclass_pos = 0;	/* position inside a character class */
 
 	p = VARDATA_ANY(pat_text);
 	plen = VARSIZE_ANY_EXHDR(pat_text);
-	pend = p + plen;
 	if (esc_text == NULL)
 	{
 		/* No ESCAPE clause provided; default to backslash as escape */
@@ -838,17 +834,6 @@ similar_escape_internal(text *pat_text, text *esc_text)
 	 * the relevant part separators in the above expansion.  If the result
 	 * of this function is used in a plain regexp match (SIMILAR TO), the
 	 * escape-double-quotes have no effect on the match behavior.
-	 *
-	 * While we don't fully validate character classes (bracket expressions),
-	 * we do need to parse them well enough to know where they end.
-	 * "charclass_pos" tracks where we are in a character class.
-	 * Its value is uninteresting when bracket_depth is 0.
-	 * But when bracket_depth > 0, it will be
-	 *   1: right after the opening '[' (a following '^' will negate
-	 *      the class, while ']' is a literal character)
-	 *   2: right after a '^' after the opening '[' (']' is still a literal
-	 *      character)
-	 *   3 or more: further inside the character class (']' ends the class)
 	 *----------
 	 */
 
@@ -882,7 +867,7 @@ similar_escape_internal(text *pat_text, text *esc_text)
 
 		if (elen > 1)
 		{
-			int			mblen = pg_mblen_range(p, pend);
+			int			mblen = pg_mblen(p);
 
 			if (mblen > 1)
 			{
@@ -920,7 +905,7 @@ similar_escape_internal(text *pat_text, text *esc_text)
 		/* fast path */
 		if (afterescape)
 		{
-			if (pchar == '"' && bracket_depth < 1)	/* escape-double-quote? */
+			if (pchar == '"' && !incharclass)	/* escape-double-quote? */
 			{
 				/* emit appropriate part separator, per notes above */
 				if (nquotes == 0)
@@ -961,12 +946,6 @@ similar_escape_internal(text *pat_text, text *esc_text)
 				 */
 				*r++ = '\\';
 				*r++ = pchar;
-
-				/*
-				 * If we encounter an escaped character in a character class,
-				 * we are no longer at the beginning.
-				 */
-				charclass_pos = 3;
 			}
 			afterescape = false;
 		}
@@ -975,69 +954,18 @@ similar_escape_internal(text *pat_text, text *esc_text)
 			/* SQL escape character; do not send to output */
 			afterescape = true;
 		}
-		else if (bracket_depth > 0)
+		else if (incharclass)
 		{
-			/* inside a character class */
 			if (pchar == '\\')
-			{
-				/*
-				 * If we're here, backslash is not the SQL escape character,
-				 * so treat it as a literal class element, which requires
-				 * doubling it.  (This matches our behavior for backslashes
-				 * outside character classes.)
-				 */
 				*r++ = '\\';
-			}
 			*r++ = pchar;
-
-			/* parse the character class well enough to identify ending ']' */
-			if (pchar == ']' && charclass_pos > 2)
-			{
-				/* found the real end of a bracket pair */
-				bracket_depth--;
-				/* don't reset charclass_pos, this may be an inner bracket */
-			}
-			else if (pchar == '[')
-			{
-				/* start of a nested bracket pair */
-				bracket_depth++;
-
-				/*
-				 * We are no longer at the beginning of a character class.
-				 * (The nested bracket pair is a collating element, not a
-				 * character class in its own right.)
-				 */
-				charclass_pos = 3;
-			}
-			else if (pchar == '^')
-			{
-				/*
-				 * A caret right after the opening bracket negates the
-				 * character class.  In that case, the following will
-				 * increment charclass_pos from 1 to 2, so that a following
-				 * ']' is still a literal character and does not end the
-				 * character class.  If we are further inside a character
-				 * class, charclass_pos might get incremented past 3, which is
-				 * fine.
-				 */
-				charclass_pos++;
-			}
-			else
-			{
-				/*
-				 * Anything else (including a backslash or leading ']') is an
-				 * element of the character class, so we are no longer at the
-				 * beginning of the class.
-				 */
-				charclass_pos = 3;
-			}
+			if (pchar == ']')
+				incharclass = false;
 		}
 		else if (pchar == '[')
 		{
-			/* start of a character class */
 			*r++ = pchar;
-			bracket_depth = 1;
-			charclass_pos = 1;
+			incharclass = true;
 		}
 		else if (pchar == '%')
 		{
