@@ -3,7 +3,7 @@
 # test parser generator for ecpg
 # call with backend grammar as stdin
 #
-# Copyright (c) 2009-2018, PostgreSQL Global Development Group
+# Copyright (c) 2009-2023, PostgreSQL Global Development Group
 #
 # Written by Michael Meskes <meskes@postgresql.org>
 #            Andy Colson <andy@squeakycode.net>
@@ -18,17 +18,20 @@
 
 use strict;
 use warnings;
-no warnings 'uninitialized';
+use Getopt::Long;
 
+my $srcdir = '.';
+my $parser = '../../../backend/parser/gram.y';
+my $stamp = '';
 my $verbose = 0;
-if ($ARGV[0] eq '-v')
-{
-	$verbose = shift;
-}
-my $path   = shift || '.';
-my $parser = shift || '../../../backend/parser/gram.y';
 
-my $filename = $path . "/ecpg.addons";
+GetOptions(
+	'srcdir=s' => \$srcdir,
+	'parser=s' => \$parser,
+	'stamp=s' => \$stamp,
+	'verbose' => \$verbose,) or die "wrong arguments";
+
+my $filename = "$srcdir/ecpg.addons";
 if ($verbose)
 {
 	print "parser: $parser\n";
@@ -39,20 +42,23 @@ my %replace_line = (
 	'ExecuteStmtEXECUTEnameexecute_param_clause' =>
 	  'EXECUTE prepared_name execute_param_clause execute_rest',
 
-	'ExecuteStmtCREATEOptTempTABLEcreate_as_targetASEXECUTEnameexecute_param_clause'
-	  => 'CREATE OptTemp TABLE create_as_target AS EXECUTE prepared_name execute_param_clause',
+	'ExecuteStmtCREATEOptTempTABLEcreate_as_targetASEXECUTEnameexecute_param_clauseopt_with_data'
+	  => 'CREATE OptTemp TABLE create_as_target AS EXECUTE prepared_name execute_param_clause opt_with_data execute_rest',
+
+	'ExecuteStmtCREATEOptTempTABLEIF_PNOTEXISTScreate_as_targetASEXECUTEnameexecute_param_clauseopt_with_data'
+	  => 'CREATE OptTemp TABLE IF_P NOT EXISTS create_as_target AS EXECUTE prepared_name execute_param_clause opt_with_data execute_rest',
 
 	'PrepareStmtPREPAREnameprep_type_clauseASPreparableStmt' =>
 	  'PREPARE prepared_name prep_type_clause AS PreparableStmt');
 
-my $block        = '';
-my $yaccmode     = 0;
-my $in_rule      = 0;
+my $block = '';
+my $yaccmode = 0;
+my $in_rule = 0;
 my $brace_indent = 0;
 my (@arr, %found);
-my $comment     = 0;
+my $comment = 0;
 my $non_term_id = '';
-my $cc          = 0;
+my $cc = 0;
 
 open my $parser_fh, '<', $parser or die $!;
 while (<$parser_fh>)
@@ -134,12 +140,14 @@ while (<$parser_fh>)
 			$block = '';
 			$in_rule = 0 if $arr[$fieldIndexer] eq ';';
 		}
-		elsif (($arr[$fieldIndexer] =~ '[A-Za-z0-9]+:')
-			|| $arr[ $fieldIndexer + 1 ] eq ':')
+		elsif (
+			($arr[$fieldIndexer] =~ '[A-Za-z0-9]+:')
+			|| (   $fieldIndexer + 1 < $n
+				&& $arr[ $fieldIndexer + 1 ] eq ':'))
 		{
 			die "unterminated rule at grammar line $.\n"
 			  if $in_rule;
-			$in_rule     = 1;
+			$in_rule = 1;
 			$non_term_id = $arr[$fieldIndexer];
 			$non_term_id =~ tr/://d;
 		}
@@ -185,4 +193,10 @@ if ($verbose)
 	print "$cc rules checked\n";
 }
 
+if ($stamp)
+{
+	open my $stampfh, '>', $stamp or die $!;
+	close $stampfh;
+}
+
 exit $ret;
