@@ -241,6 +241,7 @@ sub GenerateFiles
 		HAVE_DECL_POSIX_FADVISE => 0,
 		HAVE_DECL_PREADV => 0,
 		HAVE_DECL_PWRITEV => 0,
+		HAVE_DECL_STRCHRNUL => 0,
 		HAVE_DECL_STRLCAT => 0,
 		HAVE_DECL_STRLCPY => 0,
 		HAVE_DECL_STRNLEN => 1,
@@ -329,10 +330,9 @@ sub GenerateFiles
 		HAVE_SOCKLEN_T => 1,
 		HAVE_SPINLOCKS => 1,
 		HAVE_SSL_CTX_SET_CERT_CB => undef,
-		HAVE_STDBOOL_H => 1,
+		HAVE_SSL_CTX_SET_NUM_TICKETS => undef,
 		HAVE_STDINT_H => 1,
 		HAVE_STDLIB_H => 1,
-		HAVE_STRCHRNUL => undef,
 		HAVE_STRERROR_R => undef,
 		HAVE_STRINGS_H => undef,
 		HAVE_STRING_H => 1,
@@ -373,7 +373,6 @@ sub GenerateFiles
 		HAVE_X509_GET_SIGNATURE_NID => 1,
 		HAVE_X509_GET_SIGNATURE_INFO => undef,
 		HAVE_X86_64_POPCNTQ => undef,
-		HAVE__BOOL => undef,
 		HAVE__BUILTIN_BSWAP16 => undef,
 		HAVE__BUILTIN_BSWAP32 => undef,
 		HAVE__BUILTIN_BSWAP64 => undef,
@@ -495,6 +494,7 @@ sub GenerateFiles
 			|| ($digit1 >= '1' && $digit2 >= '1' && $digit3 >= '1'))
 		{
 			$define{HAVE_X509_GET_SIGNATURE_INFO} = 1;
+			$define{HAVE_SSL_CTX_SET_NUM_TICKETS} = 1;
 		}
 
 		# Symbols needed with OpenSSL 1.1.0 and above.
